module HmacSignature
	class Credentials
		HEADER_CREDENTIAL_REGEXES = [/^x-auth-(.+)$/i, /^http_x_auth_(.+)$/i]
		PARAM_CREDENTIAL_REGEX = /^auth_(.+)$/

		class << self
			def from_params params={}
				hash = params.inject({}) do |memo, (k,v)|
					if match = k.to_s.match(PARAM_CREDENTIAL_REGEX)
						memo[match[1]] = v
					end
					memo
				end
				new hash['key'], hash['expiry'], hash['version'], hash['signature']
			end

			def from_headers headers={}
				hash = headers.inject({}) do |memo, (k,v)|
					HEADER_CREDENTIAL_REGEXES.each do |regex|
						if match = k.match(regex)
							new_key = match[1].downcase.gsub('-', '_')
							memo[new_key] = v
							break
						end
					end
					memo
				end
				new hash['key'], hash['expiry'], hash['version'], hash['signature']
			end
		end

		attr_reader :key, :expiry, :version, :signature

		def initialize key, expiry, version, signature
			@key = key 
			@expiry = expiry
			@version = version
			@signature = signature
		end

		def to_hash
			{'auth_key' => key, 'auth_expiry' => expiry, 'auth_version' => version, 'auth_signature' => signature}
		end

		def to_headers
			{'X-Auth-Key' => key, 'X-Auth-Expiry' => expiry, 'X-Auth-Version' => version, 'X-Auth-Signature' => signature}
		end
	end
end