require 'openssl'

module HmacSignature
	class Signature < Struct.new(:method, :path, :params, :timestamp, :version, :key, :secret)
		VERSION = "1.0"

		def to_s
			digest = OpenSSL::Digest::SHA256.new
			OpenSSL::HMAC.hexdigest(digest, secret, string_to_sign)
		end

		def string_to_sign
			[method, path, parameter_string].join("\n")
		end

		private

		def credential_hash
			{ :auth_expiry => timestamp, :auth_version => version, :auth_key => key }
		end

		def parameter_string
			hash = params.merge credential_hash

			# Convert keys to lowercase strings
			hash = hash.inject({}) do |memo, (k,v)|
				memo[k.to_s.downcase] = v
				memo
			end

			hash.sort.map do |k, v|
				QueryEncoder.encode_param_without_escaping(k, v)
			end.join('&')
		end
	end
end